#!/usr/bin/env ruby
# $Id: starter_geospatial.rb,v 1.5.2.2 2016/07/28 11:13:23 rp Exp $
# Starter for georeferenced PDF:
# Import an image with a map and add geospatial reference information
#
# Sample map and coordinates:
# We use a map from www.openstreetmap.com; the geospatial coordinates of the
# image edges were also provided by that Web site.
# The coordinate system is WGS84 which is also used for GPS.
#
# Required software: PDFlib/PDFlib+PDI/PPS 9
# Required data: image file and associated geospatial reference information
#/

require './PDFlib'

# This is where the data files are. Adjust if necessary.
searchpath = "../data"
outfile = "starter_geospatial.pdf"

imagefile = "munich.png"

# WKT for plain latitude/longitude values in WGS84
georefsystem = "worldsystem={type=geographic wkt={" +
    "GEOGCS[\"WGS 84\"," +
    "  DATUM[\"WGS_1984\", SPHEROID[\"WGS 84\", 6378137,298.257223563]]," +
    "  PRIMEM[\"Greenwich\", 0.0]," +
    "  UNIT[\"Degree\", 0.01745329251994328]]" +
    "}} linearunit=M areaunit=SQM angularunit=degree"

# world coordinates of the image (in degrees)
worldpoints = [
    48.145, # latitude of top edge
    11.565, # longitude of left edge
    11.59,  # longitude of right edge
    48.13   # latitude of bottom edge
]

begin
  p = PDFlib.new

  p.set_option("SearchPath={{" + searchpath +"}}")

  # This means we must check return values of load_font() etc.
  p.set_option("errorpolicy=return")

  # all strings are expected as utf8
  p.set_option("stringformat=utf8")

  # Start the document
  if (p.begin_document(outfile, "compatibility=1.7ext3") == -1)
      raise "Error: " + p.get_errmsg()
  end

  p.set_info("Creator", "PDFlib starter sample")
  p.set_info("Title", "starter_geospatial")
  # Load the map $image 
  image = p.load_image("auto", imagefile, "")
  if (image == -1) 
    raise "Error: " + p.get_errmsg()
  end
  

  # Retrieve $image width and height 
  mapwidth = p.info_image(image, "imagewidth", "")
  mapheight = p.info_image(image, "imageheight", "")

  # Generate georeference option list 
  pageoptlist = "viewports={{ georeference={"
  pageoptlist += georefsystem.to_s + " "

  # Use the four corners as reference points; (0,0)=lower left etc.
  pageoptlist += " mappoints={0 0  1 0  1 1  0 1} "

  # The following can be used as a workaround for a problem with the
  # Avenza PDF Maps app on Android; otherwise the (almost) default
  # bounds values can be skipped:
  #
  # $pageoptlist .= "bounds={0.000001 0 0 1 1 1 1 0} "

  pageoptlist += "worldpoints={"

  # lower left corner
  pageoptlist  += worldpoints[3].to_s + " " + worldpoints[1].to_s + " "
  # lower right corner
  pageoptlist  += worldpoints[3].to_s + " " + worldpoints[2].to_s + " "
  # upper right corner
  pageoptlist  += worldpoints[0].to_s + " " + worldpoints[2].to_s + " "
  # upper left corner
  pageoptlist  += worldpoints[0].to_s + " " + worldpoints[1].to_s

  pageoptlist += "} }"

  pageoptlist += " boundingbox={0 0 "
  pageoptlist += mapwidth.to_s + " " + mapheight.to_s
  pageoptlist += "} } }"

  # Create a page with geospatial reference information. 
  p.begin_page_ext(mapwidth, mapheight, pageoptlist)
  # Place the map on the lower left corner of the page 
  optlist = "adjustpage boxsize={"
  optlist += mapwidth.to_s + " " + mapheight.to_s
  optlist += "}"
  p.fit_image(image, 0, 0, optlist)

  p.end_page_ext("")

  p.end_document("")

rescue  PDFlibException => pe
  print "PDFlib exception occurred:\n"
  print "[" + pe.get_errnum.to_s + "] " + pe.get_apiname + \
                  ": " + pe.get_errmsg + "\n"
rescue  Exception => e
  print e.backtrace.join("\n") + "\n" + e.to_s
ensure
  p.delete() if p
end

# vim: sw=2
